@@ -2,7 +2,7 @@
 #![cfg_attr(feature = "nightly", feature(min_specialization, rustc_attrs))]
 #![no_std]
 #![allow(non_snake_case)]
-#![deny(missing_docs, warnings)]
+#![deny(missing_docs)]
 #![doc = include_str!("../README.md")]
 
 #[cfg(all(feature = "alloc", not(feature = "std")))]
@@ -44,8 +44,10 @@ pub extern crate secp256k1;
 pub extern crate serde_crate as serde;
 #[cfg(feature = "libsecp_compat")]
 mod libsecp_compat;
+#[cfg(any(feature = "proptest", test))]
+mod proptest_impls;
 #[cfg(feature = "proptest")]
-pub mod proptest;
+pub extern crate proptest;
 /// The main basepoint for secp256k1 as specified in [_SEC 2: Recommended Elliptic Curve Domain Parameters_] and used in Bitcoin.
 ///
 /// At the moment, [`G`] is the only [`BasePoint`] in the library.
