@@ -1,5 +1,5 @@
 use crate::{
-    backend::{self, BackendPoint},
+    backend::{self, BackendPoint, TimeSensitive},
     hash::HashInto,
     marker::*,
     op, Scalar, XOnly,
@@ -243,6 +243,29 @@ impl Point<Normal, Public, Zero> {
     }
 }
 
+impl<T, S> Point<T, S, Zero> {
+    /// Converts a point marked with `Zero` to one that is marked `NonZero`.
+    /// You must provide a justification for this as the `reason`.
+    /// **If you're wrong the method will panic with the reason**.
+    ///
+    /// This is shorthand for:
+    ///
+    /// ```ignore
+    /// use secp256kfun::marker::*;
+    /// point.mark::<NonZero>().expect(reason);
+    /// ```
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use secp256kfun::{g, G};
+    /// let two_g = g!(G + G).expect_nonzero("2 * G is not zero");
+    /// ```
+    pub fn expect_nonzero(self, reason: &str) -> Point<T, S, NonZero> {
+        self.mark::<NonZero>().expect(reason)
+    }
+}
+
 impl<Z, T> Point<T, Public, Z> {
     /// Checks if this point's x-coordiante is the equal to the scalar mod the
     /// curve order. This is only useful for ECDSA implementations.
@@ -251,6 +274,15 @@ impl<Z, T> Point<T, Public, Z> {
     }
 }
 
+impl<S, Z> Point<Jacobian, S, Z> {
+    /// Normalize a point.
+    ///
+    /// Shorthand for calling [`.mark::<Normal>()`](crate::marker::Mark::mark).
+    pub fn normalize(self) -> Point<Normal, S, Z> {
+        self.mark::<Normal>()
+    }
+}
+
 impl<T: PointType, S, Z> core::ops::Neg for Point<T, S, Z> {
     type Output = Point<T::NegationType, S, Z>;
     fn neg(self) -> Self::Output {
@@ -270,7 +302,7 @@ impl<T: PointType, S, Z> core::ops::Neg for &Point<T, S, Z> {
 
 impl<T1, S1, Z1, T2, S2, Z2> PartialEq<Point<T2, S2, Z2>> for Point<T1, S1, Z1> {
     fn eq(&self, rhs: &Point<T2, S2, Z2>) -> bool {
-        op::PointBinary::eq((self, rhs))
+        op::PointBinary::eq(self, rhs)
     }
 }
 
@@ -310,12 +342,8 @@ impl<S, T: Normalized> Point<T, S, NonZero> {
     /// [_Standards for Efficient Cryptography_]: https://www.secg.org/sec1-v2.pdf
     /// [`from_bytes`]: crate::Point::from_bytes
     pub fn to_bytes(&self) -> [u8; 33] {
-        let mut bytes = [0u8; 33];
         let (x, y) = self.coordinates();
-        bytes[0] = y[31] & 0x01;
-        bytes[0] |= 0x02;
-        bytes[1..].copy_from_slice(&x[..]);
-        bytes
+        coords_to_bytes(x, y)
     }
 
     /// Encodes a point as its compressed encoding as specified by [_Standards for Efficient Cryptography_].
@@ -359,7 +387,7 @@ impl<S, T: Normalized> Point<T, S, NonZero> {
 }
 
 impl<T: Normalized, S> HashInto for Point<T, S, NonZero> {
-    fn hash_into(&self, hash: &mut impl digest::Digest) {
+    fn hash_into(self, hash: &mut impl digest::Digest) {
         hash.update(self.to_bytes().as_ref())
     }
 }
@@ -376,11 +404,24 @@ where
     }
 }
 
+fn coords_to_bytes(x: [u8; 32], y: [u8; 32]) -> [u8; 33] {
+    let mut bytes = [0u8; 33];
+    bytes[0] = y[31] & 0x01;
+    bytes[0] |= 0x02;
+    bytes[1..].copy_from_slice(&x[..]);
+    bytes
+}
+
 crate::impl_debug! {
-    fn to_bytes<T: PointType, S,Z>(point: &Point<T, S, Z>) -> Result<[u8;33], &str> {
-        match Clone::clone(*point).mark::<(Normal,NonZero)>() {
-            Some(nzpoint) => Ok(nzpoint.to_bytes()),
-            None => Err("Zero"),
+    fn to_bytes<T, S,Z>(point: &Point<T, S, Z>) -> Result<[u8;33], &str> {
+        let mut p = point.0.clone();
+        backend::VariableTime::point_normalize(&mut p);
+        if backend::Point::is_zero(&p) {
+            Err("Zero")
+        }
+        else {
+            let (x, y) = backend::Point::norm_to_coordinates(&p);
+            Ok(coords_to_bytes(x,y))
         }
     }
 }
@@ -417,6 +458,7 @@ crate::impl_fromstr_deserialize! {
 mod test {
     use super::*;
     use crate::{g, G};
+    use proptest::prelude::*;
     #[cfg(target_arch = "wasm32")]
     use wasm_bindgen_test::wasm_bindgen_test as test;
 
@@ -432,29 +474,31 @@ mod test {
     }
 
     macro_rules! operations_test {
-        ($P:expr) => {{
+        (@binary $P:expr, $Q:expr) => {{
             let p = $P;
+            let q = $Q;
             let i = Point::zero();
+            expression_eq!([p] == [q]);
+            expression_eq!([q] == [p]);
+            expression_eq!([1 * p] == [q]);
+            expression_eq!([-1 * p] == [-q]);
+            expression_eq!([p - q] == [i]);
+            expression_eq!([i + p] == [q]);
 
-            expression_eq!([p] == [p]);
-            expression_eq!([p] != [i]);
-            expression_eq!([p] == [p]);
-            expression_eq!([1 * p] == [p]);
-            expression_eq!([-1 * p] == [-p]);
 
-            expression_eq!([p - p] == [i]);
-            expression_eq!([i + p] == [p]);
-
-            expression_eq!([p + i] == [p]);
-            expression_eq!([i - p] == [-p]);
-            expression_eq!([p - i] == [p]);
-            expression_eq!([p + p] != [p]);
-            expression_eq!([0 * p] == [i]);
-            expression_eq!([-(p + p)] == [-p + -p]);
-            expression_eq!([p + p] == [2 * p]);
-            expression_eq!([p + p + p] == [3 * p]);
-            expression_eq!([-p - p - p] == [-3 * p]);
+            if !p.is_zero() {
+                expression_eq!([p] != [i]);
+                expression_eq!([p + p] != [p]);
+            }
 
+            expression_eq!([-(p + p)] == [-q + -q]);
+            expression_eq!([p + p] == [2 * q]);
+            expression_eq!([p + q] == [2 * q]);
+            expression_eq!([q + p] == [2 * q]);
+            expression_eq!([p + p + p] == [3 * q]);
+            expression_eq!([-p - p - p] == [-3 * q]);
+            expression_eq!([42 * p + 1337 * p] == [1379 * q]);
+            expression_eq!([42 * p - 1337 * p] == [-1295 * q]);
             let add_100_times = {
                 let p = p.clone().mark::<(Zero, Jacobian)>();
                 let i = g!(p - p);
@@ -462,29 +506,77 @@ mod test {
                 (0..100).fold(i, |acc, _| g!(acc + p))
             };
 
-            expression_eq!([add_100_times] == [100 * p]);
-
+            expression_eq!([add_100_times] == [100 * q]);
             let undo = { (0..100).fold(add_100_times.clone(), |acc, _| g!(acc - p)) };
-
-            expression_eq!([undo] == [add_100_times - 100 * p]);
+            expression_eq!([undo] == [add_100_times - 100 * q]);
             expression_eq!([undo] == [i]);
         }};
+        ($P:expr) => {{
+            let p = $P;
+            let i = Point::zero();
+
+            expression_eq!([p] == [p]);
+            expression_eq!([p + i] == [p]);
+            expression_eq!([i - p] == [-p]);
+            expression_eq!([p - i] == [p]);
+            expression_eq!([0 * p] == [i]);
+
+            let q = p.clone().mark::<(Normal, Public)>();
+            operations_test!(@binary p,q);
+            let q = p.clone().mark::<(Jacobian, Public)>();
+            operations_test!(@binary p,q);
+            let q = p.clone().mark::<(Normal, Secret)>();
+            operations_test!(@binary p,q);
+            let q = p.clone().mark::<(Jacobian, Secret)>();
+            operations_test!(@binary p,q);
+        }}
     }
 
-    #[test]
-    fn operations() {
-        operations_test!(G.clone());
-        operations_test!(G.clone().mark::<Secret>());
-        operations_test!(G.clone().mark::<(Public, Jacobian)>());
-        operations_test!(G.clone().mark::<(Secret, Jacobian)>());
-        operations_test!(Point::random(&mut rand::thread_rng()).mark::<Secret>());
-        operations_test!(Point::random(&mut rand::thread_rng()).mark::<Public>());
-        let p = crate::op::scalar_mul_point(
-            &Scalar::random(&mut rand::thread_rng()).mark::<Secret>(),
-            G,
-        );
-        operations_test!(&p);
-        operations_test!(p.mark::<Public>())
+    proptest! {
+        #[test]
+        fn operations_even_y(P in any::<Point<EvenY>>()) {
+            operations_test!(&P);
+        }
+
+        #[test]
+        fn operations_normal(P in any::<Point<Normal>>()) {
+            operations_test!(&P);
+        }
+
+        #[test]
+        fn operations_jacobian(P in any::<Point<Jacobian>>()) {
+            operations_test!(&P);
+        }
+
+        #[test]
+        fn operations_normal_secret(P in any::<Point<Normal, Secret>>()) {
+            operations_test!(&P);
+        }
+
+        #[test]
+        fn operations_jacobian_secret(P in any::<Point<Jacobian, Secret>>()) {
+            operations_test!(&P);
+        }
+
+        #[test]
+        fn operations_normal_public_zero(P in any::<Point<Normal, Public, Zero>>()) {
+            operations_test!(&P);
+        }
+
+        #[test]
+        fn operations_normal_secret_zero(P in any::<Point<Normal, Secret, Zero>>()) {
+            operations_test!(&P);
+        }
+
+        #[test]
+        fn operations_jacobian_public_zero(P in any::<Point<Jacobian, Public, Zero>>()) {
+            operations_test!(&P);
+        }
+
+        #[test]
+        fn operations_jacobian_secret_zero(P in any::<Point<Jacobian, Secret, Zero>>()) {
+            operations_test!(&P);
+        }
     }
 
     #[test]
